// g++ -D STACK_MAIN -std=c++0x Stack.cpp
// g++ -D STACK_MAIN -std=c++11 Stack.cpp


#include "Stack.hpp"

#include <cassert>
#include <cstddef>

#include <iostream>

template <typename T>
Stack<T>::Stack( void ) :
    N (0),
    list (nullptr)
    {
    }
    
template <typename T>
Stack<T>::~Stack( void ) {
    assert( N == 0 );  // Catch the potential memory leak.
    while ( list ) {
        Node * next = list->next;
        delete list;
        list = next;
        }
    }

template <typename T>
void Stack<T>::push( T &item ) {
    Node * node = new Node();
    node->item = item;
    node->next = list;
    list = node;
    N++;
    }

template <typename T>
T Stack<T>::pop( void ) {
    assert( N > 0 );

    T item = list->item;

    Node *old = list;
    list = list->next;
    delete old;
    N--;

    return item;
    }

template <typename T>
bool Stack<T>::is_empty( void ) {
    return N == 0;
    }

template <typename T>
size_t Stack<T>::size( void ) {
    return N;
    }

template <typename T>
typename Stack<T>::iterator Stack<T>::begin( void ) {
    return Stack<T>::iterator( list );
    }

template <typename T>
typename Stack<T>::iterator Stack<T>::end( void ) {
    return Stack<T>::iterator( nullptr );
    }


////////////////////////////////////////////////////////////////////////////////

template <typename T>
Stack<T>::iterator::iterator( Node *c ) :
    curr (c)
    {
    }

template <typename T>
typename Stack<T>::iterator & Stack<T>::iterator::operator++() {
    if ( this->curr != nullptr ) {
        this->curr = this->curr->next;
        }
    return *this;
    }

template <typename T>
typename Stack<T>::iterator Stack<T>::iterator::operator++( int ) {
    auto t = Stack<T>::iterator( *this );

    if ( this->curr != nullptr ) {
        this->curr = this->curr->next;
        }
    return t;
    }

template <typename T>
T Stack<T>::iterator::operator*() {
    return this->curr->item;
    }

template <typename T>
bool Stack<T>::iterator::operator!=( typename Stack<T>::iterator other ) {
    return this->curr != other.curr;
    }


////////////////////////////////////////////////////////////////////////////////

#ifdef STACK_MAIN

#include <iostream>

int main ( int argc, char **argv ) {

    Stack<long> stack;

    long i;
    while ( ! std::cin.eof() ) {
        std::cin >> i;
        if ( std::cin.good() ) 
            stack.push(i);
        }

    std::cout << "Stack has " << stack.size() << " entries." << std::endl;

    for ( auto iter = stack.begin(); iter != stack.end(); ++iter ) {
        std::cout << *iter << std::endl;
        }

    std::cout << "Popping entries..." << std::endl;

    while ( ! stack.is_empty() ) {
        i = stack.pop();
        std::cout << i << std::endl;
        }

    std::cout << "Stack has " << stack.size() << " entries." << std::endl;

    }

#endif
